﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Sailboat.Entities;

namespace Sailboat.DrawingObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawingBoat
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityBoat? EntityBoat { get; protected set; }

        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;

        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;

        /// <summary>
        /// Левая координата прорисовки лодки
        /// </summary>
        protected int _startPosX;

        /// <summary>
        /// Верхняя кооридната прорисовки лодки
        /// </summary>
        protected int _startPosY;

        /// <summary>
        /// Верхняя кооридната прорисовки лодки
        /// </summary>
        private readonly int _boatWidth = 185;

        /// <summary>
        /// Высота прорисовки лодки
        /// </summary>
        private readonly int _boatHeight = 160;

        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _boatWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _boatHeight;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawingBoat(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (width < _boatWidth || height < _boatHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityBoat = new EntityBoat(speed, weight, bodyColor);
        }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <param name="boatWidth">Ширина прорисовки лодки</param>
        /// <param name="boatHeight">Высота прорисовки лодки</param>
        protected DrawingBoat(int speed, double weight, Color bodyColor, int width, int height, int boatWidth, int boatHeight)
        {
            if (width < _boatWidth || height < _boatHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            _boatWidth = boatWidth;
            _boatHeight = boatHeight;
            EntityBoat = new EntityBoat(speed, weight, bodyColor);
        }

        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if (x < 0 || x + _boatWidth > _pictureWidth)
            {
                x = 0;
            }
            if (y < 0 || y + _boatHeight > _pictureHeight)
            {
                y = 0;
            }
            _startPosX = x;
            _startPosY = y;
        }

        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityBoat == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityBoat.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityBoat.Step > 0,
                // вправо
                DirectionType.Right => _startPosX + EntityBoat.Step < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + EntityBoat.Step < _pictureHeight,
                _ => false
            };
        }

        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityBoat == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    if (_startPosX - EntityBoat.Step > 0)
                    {
                        _startPosX -= (int)EntityBoat.Step;
                    }
                    break;
                //вверх
                case DirectionType.Up:
                    if (_startPosY - EntityBoat.Step > 0)
                    {
                        _startPosY -= (int)EntityBoat.Step;
                    }
                    break;
                //вправо
                case DirectionType.Right:
                    if (_startPosX + EntityBoat.Step + _boatWidth < _pictureWidth)
                    {
                        _startPosX += (int)EntityBoat.Step;
                    }
                    break;
                //вниз
                case DirectionType.Down:
                    if (_startPosY + EntityBoat.Step + _boatHeight < _pictureHeight)
                    {
                        _startPosY += (int)EntityBoat.Step;
                    }
                    break;
            }
        }

        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityBoat == null)
            {
                return;
            }
            Pen pen = new(Color.Black);

            // Основной корпус парусника 
            Brush Brush = new
            SolidBrush(EntityBoat.BodyColor);

            Point[] hull = new Point[]
            {
                new Point(_startPosX + 5, _startPosY + 90),
                new Point(_startPosX + 120, _startPosY + 90),
                new Point(_startPosX + 180, _startPosY + 120),
                new Point(_startPosX + 120, _startPosY + 150),
                new Point(_startPosX + 5, _startPosY + 150)
            };
            g.FillPolygon(Brush, hull);
            g.DrawPolygon(pen, hull);

            Brush addBrush = new
            SolidBrush(Color.Green);

            g.FillEllipse(addBrush, _startPosX + 20, _startPosY + 100, 100, 40);
            g.DrawEllipse(pen, _startPosX + 20, _startPosY + 100, 100, 40);
        }
    }
}
